@@ -15,11 +15,19 @@
 #include "intincl.h"
 
 #include "stdlib.h"
+#include <sys/types.h>
+
+struct at_hw_access {
+  char* (*read) (int (*stop_condition)(const char*, const size_t));
+  int (*write) (const char* data, size_t count);
+};
 
 /** MUST be run before any other at_* function is used
+ * @param tty must not be NULL, specifies how to access the tty
  * @param pin may be NULL, PIN to use _once_ in automatic pin handling
  */
-void at_init (char* pin);
+void at_init (struct at_hw_access* tty, char* pin);
+void at_set_tty(struct at_hw_access* tty);
 
 /** test if the vendor is supported and model not blacklisted
  */
@@ -46,6 +54,9 @@ void at_command_send (const char* command, const char* parmlist);
  */
 void at_syntax_request (const char* command);
 
+/** may be used to send data, e.g. requested by an "> " prompt
+ */
+void at_data_send (const char* data, size_t size);
 
 /** evaluate a return value (a line returned by the phone)
  * @param reponse the reponse line
