@@ -173,6 +173,18 @@ struct charset_translation gsm_chars[] = {
   { 0x0000, { 0x00, 0x00 }, 0 }
 };
 
+#include <string.h>
+size_t gsmwidth (const  gsmchar_t* input) {
+  size_t count = strlen((char*)input);
+  unsigned int i = 0;
+
+  if (input == NULL) return 0;
+  for (; input[i] != 0; ++i)
+    /* 0x1b marks an extension table */
+    if (input[i] == 0x1b) --count;
+  return count;
+}
+
 unsigned int gsm_count (ucs4char_t* input, unsigned int count) {
   unsigned int i = 0;
   unsigned int k = 0;
