@@ -18,11 +18,10 @@
 #include <iconv.h>
 #include <string.h>
 
-#include <config.h>
 #ifdef WORDS_BIGENDIAN
-#  define UCS2_WITH_ENDIANESS "UCS-2BE"
+#define UCS2_DEF "UCS-2BE"
 #else
-#  define UCS2_WITH_ENDIANESS "UCS-2LE"
+#define UCS2_DEF "UCS-2LE"
 #endif
 
 const char* ucs2_get_iconv_charset() {
@@ -30,12 +29,12 @@ const char* ucs2_get_iconv_charset() {
   iconv_t cd;
 
   if (ucs2_charset == NULL) {
-    cd = iconv_open(UCS2_WITH_ENDIANESS,UCS2_WITH_ENDIANESS);
+    cd = iconv_open(UCS2_DEF,UCS2_DEF);
     if (cd == (iconv_t)-1) {
-      fprintf(stderr,_("%s: the iconv implementation of this system does not support %s"),_("Error"),UCS2_WITH_ENDIANESS);
+      fprintf(stderr,_("%s: the iconv implementation of this system does not support %s"),_("Error"),UCS2_DEF);
       exit(EXIT_FAILURE);
     } else {
-      ucs2_charset = UCS2_WITH_ENDIANESS;
+      ucs2_charset = UCS2_DEF;
     }
     iconv_close(cd);
   }
