@@ -24,7 +24,8 @@ ucs4char_t* convert_from_ucs2 (ucs2char_t* input) {
 			     ucs2len(input)*sizeof(ucs2char_t));
 }
 
-ucs4char_t* convert_from_ucs2_hexstring (char* input) {
+ucs4char_t* convert_from_ucs2_hexstring (char* input,
+					 uint16_t (*tohe)(uint16_t i)) {
   ucs4char_t* retval;
   ucs2char_t* temp;
   unsigned int i;
@@ -32,7 +33,7 @@ ucs4char_t* convert_from_ucs2_hexstring (char* input) {
   if (input == NULL) return NULL;
   temp = mem_alloc(((strlen(input)/4)+1)*sizeof(ucs2char_t),1);
   for (i = 0; ((i+1)*4) <= strlen(input); ++i) {
-    temp[i]=hexstr2int(input+(i*4),4);
+    temp[i]=tohe(hexstr2int(input+(i*4),4));
   }
 
   retval = convert_from_ucs2(temp);
