@@ -54,14 +54,14 @@ ucs4char_t* ucs4ndup(const ucs4char_t* s, size_t n) {
   size_t i = 0;
 
   if (s == NULL) return NULL;
-  while (s[i] != 0 && i <= n) ++i;
+  while (s[i] != 0 && i < n) ++i;
   retval = mem_alloc((i+1)*sizeof(*retval),0);
   (void)memcpy(retval,s,i*sizeof(*s));
   retval[i] = 0;
   return retval;
 }
 ucs4char_t* ucs4dup(const ucs4char_t* s) {
-  return ucs4ndup((ucs4char_t*)s,ucs4len(s));
+  return ucs4ndup((ucs4char_t*)s,ucs4len(s)+1);
 }
 
 int ucs4ncmp (const ucs4char_t* s1, const ucs4char_t* s2, size_t n) {
@@ -103,3 +103,12 @@ ucs4char_t* ucs4str (const ucs4char_t* haystack,
   }
   return NULL;
 }
+
+ucs4char_t* ucs4ncpy (ucs4char_t* dest,
+		      const ucs4char_t* src,
+		      size_t n) {
+  size_t i = 0;
+  for (; i<n; ++i) dest[i] = src[i];
+  dest[i] = 0;
+  return dest;
+}
