@@ -23,9 +23,6 @@
 #include <iconv.h>
 #include <limits.h>
 
-#ifndef MB_LEN_MAX
-#define MB_LEN_MAX 6
-#endif
 
 static
 size_t replace_char_escape (char **inbuf, size_t *inbytesleft,
@@ -98,17 +95,23 @@ char* convert_from_internal (const char* to_code,
   iconv_t cd;
   size_t status;
   int estatus;
-
-  size_t insize;
-  size_t insize_conv;
-  char* inptr;
-  char* inptr_conv;
-
   char* retval;
+
+  size_t insize = sizeof(ucs4char_t);
+  size_t insize_conv;
+  char* inptr = (char*)input;
+  char* inptr_conv = inptr;
+
+  /* must be at least 5 for replace_char functions */
+  const size_t mult = (MB_LEN_MAX>5)?MB_LEN_MAX:5;
+
+  /* this should be enough for every possible locale
+   * MB_CUR_MAX can NOT be used!
+   */
+  size_t outsize = ucs4len(input)*mult;
+  size_t outsize_conv = outsize;
   char* outptr;
   char* outptr_conv;
-  size_t outsize;
-  size_t outsize_conv;
 
   if (to_code == NULL || input == NULL) return NULL;
   
@@ -119,14 +122,7 @@ char* convert_from_internal (const char* to_code,
     exit(EXIT_FAILURE);
   }
 
-  insize = sizeof(ucs4char_t);
-  inptr = (char*)input;
-  inptr_conv = inptr;
-
-  //this should be enough for every possible locale  
-  outsize = ucs4len(input)*MB_LEN_MAX;
-  outsize_conv = outsize;
-  retval = mem_alloc(outsize+1,1); //not to be modified later
+  retval = mem_alloc(outsize+mult,1); //not to be modified later
   outptr = retval;
   outptr_conv = retval;
 
