@@ -19,13 +19,13 @@
  * does not work with Cygwin (log10l() not implemented)
  */
 #include <config.h>
-#if defined(HAVE_MATH_H) && defined(HAVE_LOG10L) && defined(HAVE_LABS)
+#if defined(HAVE_MATH_H) && defined(HAVE_LOG10) && defined(HAVE_LABS)
 #include <math.h>
 unsigned int numlen (long num) {
   unsigned int a = 1;
   if (num == 0) return 1;
   else if (num < 0) ++a;
-  return (unsigned int)(log10l((long double)labs(num))+a);
+  return (unsigned int)(log10((double)labs(num))+a);
 }
 #else
 unsigned int numlen (long num) {
@@ -43,16 +43,6 @@ unsigned int numlen (long num) {
 }
 #endif
 
-uint8_t numtype (const char* number) {
-  //see ETSI 23.040, Ch. 9.1.2.5
-  if (str_len(number) == 0 ||
-      *number != '+') {
-    return 0x81;
-  } else {
-    return 0x91;
-  }
-}
-
 unsigned int hexstr2int (char *hexstring, unsigned int length) {
   unsigned int i = 0;
   unsigned int intvalue = 0;
@@ -109,3 +99,46 @@ int is_telephone_number (const char* s) {
   if (s[0] == '+') ++s;
   return is_number(s);
 }
+
+uint16_t letohs (uint16_t i) {
+  uint8_t* k = (uint8_t*)&i;
+  return (uint16_t)(k[0]|(k[1]<<8));
+}
+
+uint32_t letohl (uint32_t i) {
+  uint8_t* k = (uint8_t*)&i;
+  return (uint32_t)(k[0]|(k[1]<<8)|(k[2]<<16)|(k[3]<<24));
+}
+
+#include <assert.h>
+uint16_t htoles (uint16_t i) {
+  union {
+    struct __attribute__((packed)) {
+      uint8_t low;
+      uint8_t high;
+    } s;
+    uint16_t n;
+  } u;
+  assert(sizeof(u) == sizeof(uint16_t));
+  u.s.low = i&0xFF;
+  u.s.high = (i>>8)&0xFF;
+  return u.n;  
+}
+
+uint32_t htolel (uint32_t i) {
+  union {
+    struct __attribute__((packed)) {
+      uint8_t lowest;
+      uint8_t low;
+      uint8_t high;
+      uint8_t highest;
+    } s;
+    uint32_t n;
+  } u;
+  assert(sizeof(u) == sizeof(uint32_t));
+  u.s.lowest = i&0xFF;
+  u.s.low = (i>>8)&0xFF;
+  u.s.high = (i>>16)&0xFF;
+  u.s.highest = (i>>24)&0xFF;
+  return (uint32_t)u.n;
+}
