@@ -30,35 +30,32 @@
 #include <ctype.h>
 
 static
-char* sms_pdu_create_submit_frame (char* number,
+struct sms_pdu_raw* sms_pdu_create_submit_frame (char* number,
 				   unsigned long options,
 				   int header_present)
 {
   uint8_t sca_len = 0;
-  struct sms_number sca; //short message service center address
+  struct gsm_number sca; //short message service center address
   uint8_t pdutype; //PDU type
   uint8_t mr; //message reference
   uint8_t da_len = 0;
-  struct sms_number da; //destination address
+  struct gsm_number da; //destination address
   uint8_t pid = 0; //protocol identifier (normally 0)
   uint8_t dcs; //data coding scheme
   uint8_t vp_rel = 0xff; //validity period (0xff = max = 63 weeks)
   //char vp_abs[15]; //validity period (YYMMDDhhmmssTZ)
 
-  unsigned int i;
-  unsigned int k;
-  char* frame = mem_alloc(176*2+1,1);
+  struct sms_pdu_raw* frame = mem_alloc(sizeof(*frame),0);
   char* temp;
+  uint8_t* ptr;
+
+  sms_pdu_raw_init(frame);
 
   /* not yet supported
    * zero length means that the phone will use its default
    */
-  sms_number_init(&sca);
-  sms_number_set(&sca,numtype(sca.digits),NULL);
-  sca_len = strlen(sca.digits)/2;
-  if (sca_len) {
-    ++sca_len;
-  }
+  gsm_number_init(&sca);
+  sca_len = 0;
   
   /* set PDU type to standard values
    */
@@ -81,9 +78,10 @@ char* sms_pdu_create_submit_frame (char* number,
     fprintf(stderr,"%s: %s\n",_("Error"),_("No valid number specified."));
     return NULL;
   }
-  sms_number_init(&da);
-  sms_number_set(&da,numtype(number),number);
-  temp = sms_number_get(&da);
+  gsm_number_init(&da);
+  gsm_number_set_toa(&da,numtype(number));
+  gsm_number_set(&da,number,str_len(number));
+  temp = gsm_number_get(&da);
   if (temp == NULL || strcmp(temp,number) != 0) {
     fprintf(stderr,_("%s: sms number cannot have more than %lu digits.\n"),_("Error"),(unsigned long)sizeof(da.digits)-1);
     return NULL;
@@ -106,40 +104,47 @@ char* sms_pdu_create_submit_frame (char* number,
   //leave pid and vp_rel as is
 
   //create PDU frame
-  sprintf(frame,"%02X",sca_len);
+  ptr = frame->data;
+  ptr[0] = sca_len;
+  ++ptr;
   if (sca_len) {
-    sprintf(frame+2,"%02X%s",sms_number_get_toa(&sca),sca.digits);
+    ptr[0] = gsm_number_get_toa(&sca);
+    ptr += gsm_number_get_semioctets(&sca,ptr)+1;
   }
-  sprintf(frame+strlen(frame),"%02X%02X%02X%02X",
-	  pdutype,mr,da_len,sms_number_get_toa(&da));
-  k = strlen(da.digits);
-  for (i = 0; i < k; i += 2) {
-    sprintf(frame+strlen(frame),"%c%c",
-	    (i+1>=k && k%2) ? 'F' : da.digits[i+1],
-	    da.digits[i]);
-  }
-  sprintf(frame+strlen(frame),"%02X%02X%02X",
-	  pid,dcs,vp_rel);
 
-  sms_number_delete(&sca);
-  sms_number_delete(&da);
+  ptr[0] = pdutype;
+  ptr[1] = mr;
+  ptr += 2;
+  
+  ptr[0] = da_len;
+  ptr[1] = gsm_number_get_toa(&da);
+  ptr += 2;
+  if (da_len) ptr += gsm_number_get_semioctets(&da,ptr);
+
+  ptr[0] = pid;
+  ptr[1] = dcs;
+  ptr[2] = vp_rel;
+  ptr += 3;
+  frame->size = ptr-frame->data;
+
+  gsm_number_delete(&sca);
+  gsm_number_delete(&da);
 
   return frame;
 }
 
-char** sms_pdu_create_submit (char* text,
-			      char* number,
-			      unsigned long options)
+struct sms_pdu_raw** sms_pdu_create_submit (char* text,
+					    char* number,
+					    unsigned long options)
 {
   ucs4char_t* wide_str = convert_from_system(text);
-  char** pdu = NULL;
+  struct sms_pdu_raw** tpdu = NULL;
   unsigned int i = 0;
-  unsigned int k;
-  char* frame;
+  struct sms_pdu_raw* frame;
   enum sms_encoding charset = SMS_CHARSET_GSM;
   gsmchar_t* g1 = NULL;
   ucs2char_t* g2 = NULL;
-  
+
   if (options&SMS_CREATE_OPT_UNICODE) charset = SMS_CHARSET_UCS2;
   if (charset == SMS_CHARSET_GSM &&
       options&SMS_CREATE_OPT_AUTOCHAR)
@@ -166,22 +171,21 @@ char** sms_pdu_create_submit (char* text,
     }
     mem_realloc(g1,0);
   }
-  pdu = sms_data_encode(charset,NULL, wide_str);
+  tpdu = sms_data_encode(charset,NULL, wide_str);
   mem_realloc(wide_str,0);
 
-  if (pdu == NULL) return NULL;
+  if (tpdu == NULL) return NULL;
 
   frame = sms_pdu_create_submit_frame(number,options,
-				      (pdu[1] == NULL)? 0 : 1);
+				      (tpdu[1] == NULL)? 0 : 1);
   if (frame == NULL) return NULL;
 
-  k = strlen(frame);
-  for (; pdu[i] != NULL; ++i) {
-    pdu[i] = mem_realloc(pdu[i],(176*2)+1);
-    memmove(pdu[i]+k,pdu[i],strlen(pdu[i])+1);
-    memcpy(pdu[i],frame,k);
+  for (; tpdu[i] != NULL; ++i) {
+    memmove(tpdu[i]->data+frame->size,tpdu[i]->data,tpdu[i]->size);
+    memcpy(tpdu[i]->data,frame->data,frame->size);
+    tpdu[i]->size += frame->size;
   }
   mem_realloc(frame,0);
 
-  return pdu;
+  return tpdu;
 }
