@@ -61,7 +61,7 @@ void sms_pdu_print_item_address (FILE* fp, struct sms* sms) {
     }
     fprintf(fp,"%s:",temp);
     if (str_len(temp) > 0) {
-      temp = sms_number_get(&pdu->address);
+      temp = gsm_number_get(&pdu->address);
       if (temp != NULL) fprintf(fp," %s",temp);
       mem_realloc(temp,0);
     }
@@ -127,7 +127,7 @@ void sms_pdu_print_item_time (FILE* fp, enum sms_pdu_type type,
 
 static
 void sms_pdu_print_item_smsc (FILE* fp, struct sms* sms) {
-  char* temp = sms_number_get(&(sms->decoded->sca));
+  char* temp = gsm_number_get(&(sms->decoded->sca));
   if (str_len(temp) > 0)
     fprintf(fp,"%s: %s\n",_("SMSC number"),temp);
   mem_realloc(temp,0);
@@ -151,13 +151,22 @@ void sms_pdu_print_item_options (FILE* fp, struct sms* sms) {
 
   pdu = sms->decoded->pdu;
   fprintf(fp,"%s: %s\n",_("PDU type"),sms_pdu_type_string(pdu->options.type));
-  if (pdu->options.mms || pdu->options.rd ||
-      pdu->options.rp || pdu->options.sr) {
+  /* since this is a union and deliver is the biggest... */
+  if (pdu->options.type != SMS_TYPE_SUBMIT_REPORT &&
+      pdu->options.type != SMS_TYPE_DELIVER_REPORT &&
+      (pdu->options.u.deliver.sr || pdu->options.u.deliver.mms ||
+       pdu->options.u.deliver.rp)) {
     fprintf(fp,"%s:",_("PDU flags"));
-    if (pdu->options.rp) fprintf(fp," %s",_("ReplyPath"));
-    if (pdu->options.sr) fprintf(fp," %s",_("StatusRequest"));
-    if (pdu->options.mms) fprintf(fp," %s",_("MoreMessagesToSend"));
-    if (pdu->options.rd) fprintf(fp," %s",_("RejectDuplicate"));
+    if (pdu->options.u.deliver.sr) fprintf(fp," %s",_("StatusRequest"));
+    if (pdu->options.type == SMS_TYPE_SUBMIT) {
+      if (pdu->options.u.submit.rd) fprintf(fp," %s",_("RejectDuplicate"));
+    } else if (pdu->options.type == SMS_TYPE_DELIVER ||
+	       pdu->options.type == SMS_TYPE_STATUS_REPORT) {
+      if (pdu->options.u.deliver.mms) fprintf(fp," %s",_("MoreMessagesToSend"));
+    }
+    if (pdu->options.type == SMS_TYPE_DELIVER ||
+	pdu->options.type == SMS_TYPE_SUBMIT)
+      if (pdu->options.u.deliver.rp) fprintf(fp," %s",_("ReplyPath"));
     fprintf(fp,"\n");
   }
 }
