@@ -33,8 +33,8 @@ uint16_t sms_new_sequence_id () {
 }
 
 static
-unsigned int sms_header_size (struct sms_pdu_ud_header** header) {
-  unsigned int headersize = 1;
+uint8_t sms_header_size (struct sms_pdu_ud_header** header) {
+  uint8_t headersize = 1;
   unsigned int i = 0;
 
   if (header == NULL) {
@@ -49,57 +49,52 @@ unsigned int sms_header_size (struct sms_pdu_ud_header** header) {
 
 static
 void sms_data_insert_header (struct sms_pdu_ud_header** header,
-			     char* userdata)
+			     struct sms_pdu_raw* userdata)
 {
-  unsigned int size = sms_header_size(header);
+  uint8_t size = sms_header_size(header);
   unsigned int i = 0;
-  unsigned int k = 2;
-  unsigned int l = 0;
-  char* temp = NULL;
 
-  if (userdata == NULL || size == 0)
-    return;
-  temp = mem_alloc(2*size+1,1);
-  sprintf(temp,"%02X",size-1);
+  if (userdata == NULL || size == 0) return;
+  --size;
+
+  sms_pdu_raw_append(userdata,&size,1);
   for (; header[i] != NULL; ++i) {
-    sprintf(temp + k,"%02X",header[i]->type);
-    sprintf(temp + k + 2,"%02X",header[i]->len);
-    for (; l < header[i]->len; ++l)
-      sprintf(temp + k + 4 + 2*l,"%02X",header[i]->data[l]);
-    k += 4 + 2*header[i]->len;
+    sms_pdu_raw_append(userdata,&header[i]->type,1);
+    sms_pdu_raw_append(userdata,&header[i]->len,1);
+    sms_pdu_raw_append(userdata,header[i]->data,header[i]->len);
   }
-  memcpy(userdata,temp,2*size);
-  mem_realloc(temp,0);
 }
 
 static
-char* sms_data_gsm_encode (struct sms_pdu_ud_header** header,
-			   ucs4char_t* input)
+struct sms_pdu_raw* sms_data_gsm_encode (struct sms_pdu_ud_header** header,
+					 ucs4char_t* input)
 {
-  char* retval;
+  struct sms_pdu_raw* retval;
   gsmchar_t* temp;
   unsigned char first, end;
 
   unsigned int headersize = sms_header_size(header);
   unsigned int enc_len = 0;
   unsigned int inc = 0;
-  unsigned int outc = headersize*2;
-  unsigned int state = (headersize*8)/7;
+  uint8_t state = (headersize*8)/7;
+  uint8_t value = 0;
   
   if (headersize > 140) return NULL;
 
   temp = convert_to_gsm(input);
   enc_len = strlen((char*)temp);
-  retval = mem_alloc(2 + (140*2) + 1, 1);
-  sms_data_insert_header(header,retval+2);
+  retval = mem_alloc(sizeof(*retval),0);
+  sms_pdu_raw_init(retval);
+  retval->size = 1;
+  sms_data_insert_header(header,retval);
 
   if (ucs4len(input) > 0) {
     /* 7bit-GSM octet encoding
      */
     if (state%8 != 0) { //state%8 == 0 equals headersize%7 == 0
       // do the septet padding
-      sprintf(retval+2 + outc, "%02X", ((temp[inc] & 0x7F) << (7-(state%8))) & 0xFF);
-      outc += 2;
+      value = ((temp[inc] & 0x7F) << (7-(state%8))) & 0xFF;
+      sms_pdu_raw_append(retval,&value,1);
       ++state;
     }
     for (; inc < enc_len && state < (140*8/7); ++inc, ++state) {
@@ -108,9 +103,8 @@ char* sms_data_gsm_encode (struct sms_pdu_ud_header** header,
 	//shifting dependent on state (0-7)
 	first = ((temp[inc+1] & 0x7F) << (7-(state%8))) & 0xFF;
 	end = (temp[inc] & 0x7F) >> (state%8);
-	//storing as hexstring
-	sprintf(retval+2 + outc, "%02X", (first | end));
-	outc += 2;
+	value = first|end;
+	sms_pdu_raw_append(retval,&value,1);
       }
     }
 
@@ -125,45 +119,45 @@ char* sms_data_gsm_encode (struct sms_pdu_ud_header** header,
   /* the user data length must contain the septet count
    * of all userdata (including header)
    */
-  mem_realloc(temp,3);
-  sprintf((char*)temp,"%02X",state);
-  memcpy(retval,temp,2);
-  mem_realloc(temp,0);
+  retval->data[0] = state;
 
   return retval;
 }
 
 static
-char* sms_data_ucs2_encode (struct sms_pdu_ud_header** header,
+struct sms_pdu_raw* sms_data_ucs2_encode (struct sms_pdu_ud_header** header,
 			    ucs4char_t* input)
 {
-  char* retval;
-  char* tmp;
-  unsigned int enc_len = 0;
-  unsigned int headersize = sms_header_size(header);
+  struct sms_pdu_raw* retval;
+  ucs2char_t* tmp;
+  uint8_t enc_len = 0;
+  uint8_t headersize = sms_header_size(header);
   
   if (headersize > 140) return NULL;
 
-  tmp = convert_to_ucs2_hexstring(input);
-  enc_len = strlen(tmp)/2;
-  retval = mem_alloc((141*2)+1,1);
+  tmp = convert_to_ucs2(input);
+  /* convert to big-endian */
+  if (tmp) for (; tmp[enc_len] != 0; ++enc_len)
+    tmp[enc_len] = htobes(tmp[enc_len]);
+  enc_len *= 2;
+  retval = mem_alloc(sizeof(*retval),1);
+  sms_pdu_raw_init(retval);
+  retval->size = 1;
 
-  if (headersize) sms_data_insert_header(header,retval+2);
-  memcpy(retval+2+strlen(retval+2),tmp,strlen(tmp));
-  mem_realloc(tmp,3);
-  sprintf(tmp,"%02X",(unsigned int)((strlen(retval+2)/2)&0xFF));
-  memcpy(retval,tmp,2);
-  mem_realloc(tmp,0);
+  if (headersize) sms_data_insert_header(header,retval);
+  sms_pdu_raw_append(retval,(uint8_t*)tmp,(size_t)enc_len);
+  enc_len = retval->size-1;
+  retval->data[0] = enc_len;
 
   return retval;
 }
 
 #define SMS_PARTS_SUGGESTED_MAX 10
-char** sms_data_encode (enum sms_encoding charset,
-			struct sms_pdu_ud_header** header,
-			ucs4char_t* input)
+struct sms_pdu_raw** sms_data_encode (enum sms_encoding charset,
+				      struct sms_pdu_ud_header** header,
+				      ucs4char_t* input)
 {
-  char** retval = NULL;
+  struct sms_pdu_raw** retval = NULL;
 
   struct sms_pdu_ud_header** h = NULL;
   unsigned int hcount = 0;
@@ -243,7 +237,7 @@ char** sms_data_encode (enum sms_encoding charset,
 	    _("Error"));
     fprintf(stderr,_("%s: SMS text is too long (max. %d characters).\n"),
 	    _("Error"),
-	    (charset==SMS_CHARSET_GSM) ? 255*140*8/7 : 255*140/2);
+	    (charset==SMS_CHARSET_GSM) ? 255*152 : 255*66);
     if (charset == SMS_CHARSET_GSM) {
       fprintf(stderr,"%s\n",
 	      _("Please be aware that some characters "
