@@ -18,6 +18,7 @@
 
 //standard headers
 #include <stdio.h>
+#include <string.h>
 
 struct sms_pdu_ud_header*
 sms_udh_multipart16_get (uint16_t seqnum,
@@ -153,12 +154,11 @@ void sms_udh_print (FILE* fp, struct sms_pdu_ud* plist, uint8_t parts) {
   }
 }
 
-void sms_udh_fill (struct sms_pdu* sms, char* data) {
+void sms_udh_fill (struct sms_pdu* sms, uint8_t* data) {
   uint8_t udhsize = 0;
   uint8_t offset = 0;
   unsigned int i = 0;
   unsigned int k;
-  unsigned int count;
   struct sms_pdu_ud_header** header;
 
   if (sms == NULL) {
@@ -172,19 +172,19 @@ void sms_udh_fill (struct sms_pdu* sms, char* data) {
 
   if (sms->options.udh_present) {
     //extract the user data header data
-    udhsize = hexstr2int(data,2);
-    data += 2;
+    udhsize = data[0];
+    data += 1;
     //split the udh into its parts
     sms->ud[0].header = mem_alloc(((udhsize/2)+1)*sizeof(*sms->ud[0].header),0);
     header = sms->ud[0].header;
     header[i] = NULL;
-    while (offset+4 <= udhsize*2) {
+    while (offset+2 <= udhsize) {
       header[i] = mem_alloc(sizeof(**header),0);
       sms_pdu_ud_header_init(header[i]);
       //read fields that are mandatory for each user data header
-      header[i]->type = hexstr2int(data+offset,2)&0xFF;
-      header[i]->len = hexstr2int(data+2+offset,2)&0xFF;
-      offset += 4;
+      header[i]->type = data[offset];
+      header[i]->len = data[offset+1];
+      offset += 2;
       //find the udh type in the list of known headers
       k = 0;
       while (sms_pdu_ud_header_defs[k].type != -1 &&
@@ -192,16 +192,14 @@ void sms_udh_fill (struct sms_pdu* sms, char* data) {
 	++k;
 
       //check that everything is correct with the udh data
-      if (offset+(header[i]->len*2) <= udhsize*2 &&
+      if (offset+(header[i]->len) <= udhsize &&
 	  sms_pdu_ud_header_defs[k].type != -1 &&
 	  sms_pdu_ud_header_defs[k].minlen <= header[i]->len &&
 	  header[i]->len <= sms_pdu_ud_header_defs[k].maxlen)
       {
-	for (count = 0; count < header[i]->len; ++count) {
-	  header[i]->data[count] = hexstr2int(data+offset+2*count,2)&0xFF;
-	}
+	memcpy(header[i]->data,data+offset,header[i]->len);
 	header[i+1] = NULL;
-	offset += 2*header[i]->len;
+	offset += header[i]->len;
       } else {
 	//bogus or unknown header, delete it and stop processing the headers
 	mem_realloc(header[i],0);
