@@ -34,7 +34,7 @@ void sms_udh_print_header (FILE* fp, struct sms_pdu_ud_header* header);
 
 /** to parse the user data header from a pdu
  */
-void sms_udh_fill (struct sms_pdu* sms, char* data);
+void sms_udh_fill (struct sms_pdu* sms, uint8_t* data);
 
 /** called by sms_udh_fill(), no need to call anywhere else
  */
