@@ -27,7 +27,7 @@
 /* TODO: this entry is not correct */
 #define ETIMEDOUT 116
 
-#define PATH_SEPERATOR '\\'
+#define PATH_SEPARATOR '\\'
 
 #endif
 
